# charlist 1 with all small letters, numbers, some special chars and
# some capital letters chosen after their frequency.
# low frequency CAPITAL letters in both eng/ger are:
#   X Q Z U J Y O V L  (lowest to higher freq)
#   J L O Q U V X Y Z  (alphabetical)

# ASCII not present:
#      00 nul   01 soh   02 stx   03 etx   04 eot   05 enq   06 ack   07 bel
#      08 bs    09 ht    0a nl    0b vt    0c np    0d cr    0e so    0f si
#      10 dle   11 dc1   12 dc2   13 dc3   14 dc4   15 nak   16 syn   17 etb
#      18 can   19 em    1a sub   1b esc   1c fs    1d gs    1e rs    1f us

#                                                                     7f del

# more changes 14.10.2023:
# - detect '__NULL__', '__use_charlist64_2__' and '__use_UTF8__' in de-/encoding functions, not by index
# - make charlist definitions more easy to change
charlist64_1_v3=(
	                                  # dez     hex     oct     b64
	"A" "B" "C" "D" "E" "F" "G" "H"   # 00..07  00..07  00..07  A..H
	"I" "J" "K" "L" "M" "N" "O" "P"   # 08..15  08..0f  10..17  I..P
	"," "R" "S" "T"                   # 16.19.  10.13.  20.23.  Q.T.
	         "__use_UTF8__"           # ..20..  ..14..  ..24..  .U..  ("UTF8 en/decoding")
	                    "." "W"       # 21.22.  15.16.  25.26.  .VW.
	         "__use_charlist64_2__"   # ....23  ....17  ....27  ...X  ("Xtended charlist")
	"?" "Z" "a" "b" "c" "d" "e" "f"   # 24..31  18..1f  30..37  Y..f
	"g" "h" "i" "j" "k" "l" "m" "n"   # 32..39  20..27  40..47  g..n
	"o" "p" "q" "r" "s" "t" "u" "v"   # 40..47  28..2f  50..57  o..v
	"w" "x" "y" "z" "0" "1" "2" "3"   # 48..55  30..37  60..67  w..3
	"4" "5" "6" "7" "8" "9" " "       # 56.61.  38.3d.  70.75.  4.+.
	"__NULL__"                        # ....63  ....3f  ....77  .../
)
charlist64_2_v3=(
	                                  # dez     hex     oct
	"Ä" "&" "$" "Ⓐ" "€" "ß" "À" "\""  # 00..07  00..07  00..07
	"Ü" "(" "#" "*" "%" "'" ")" "-"   # 08..15  08..0f  10..17
	"Q" ";" "<" "=" "U" "V" ">" "X"   # 16..23  10..17  20..27
	"Y" ":" "ä" "@" "[" "\\" "é" "]"  # 24..31  18..1f  30..37
	"^" "_" "í" "\`" "{" "|" "}" "~"  # 32..39  20..27  40..47
	"ö" "Á" "á" "à" "è" "ø" "ü" "Ø"   # 40..47  28..2f  50..57
	"ú" "ù" "ç" "Ç" "Ö" "!" "É" "È"   # 48..55  30..37  60..67
	"Í" "Ú" "Ù" "°" "∑" "©" "+" "/"   # 56..61  38..3d  70..75
)

# new layout, matching most possible base64 charset encodings
charlist64_1_v2=(
	                                  # dez     hex     oct
	"__NULL__"                        # 0       00      00
	    "B" "C" "D" "E" "F" "G" "H"   # 01..07  01..07  01..07
	"I" "?" "K" "*" "M" "N" "." "P"   # 08..15  08..0f  10..17
	"," "R" "S" "T" "(" ")" "W" "A"   # 16..23  10..17  20..27
	" " ":" "a" "b" "c" "d" "e" "f"   # 24..31  18..1f  30..37
	"g" "h" "i" "j" "k" "l" "m" "n"   # 32..39  20..27  40..47
	"o" "p" "q" "r" "s" "t" "u" "v"   # 40..47  28..2f  50..57
	"w" "x" "y" "z" "0" "1" "2" "3"   # 48..55  30..37  60..67
	"4" "5" "6" "7" "8" "9"           # 56..61  38..3d  70..75
	"__use_charlist64_2__"            # 62      3e      76
	"__use_UTF8__"                    # 63      3f      77
)
# charlist extension to get all printable ASCII chars and some UTF8
charlist64_2_v2=(
	                                  # dez     hex     oct
	"Ä" "&" "$" "Ⓐ" "€" "ß" "À" "\""  # 00..07  00..07  00..07
	"Ü" "J" "#" "L" "%" "'" "O" "-"   # 08..15  08..0f  10..17
	"Q" ";" "<" "=" "U" "V" ">" "X"   # 16..23  10..17  20..27
	"Y" "Z" "ä" "@" "[" "\\" "é" "]"  # 24..31  18..1f  30..37
	"^" "_" "í" "\`" "{" "|" "}" "~"  # 32..39  20..27  40..47
	"ö" "Á" "á" "à" "è" "ø" "ü" "Ø"   # 40..47  28..2f  50..57
	"ú" "ù" "ç" "Ç" "Ö" "!" "É" "È"   # 48..55  30..37  60..67
	"Í" "Ú" "Ù" "°" "∑" "©" "+" "/"   # 56..61  38..3d  70..75
)
charlist64_1=( "${charlist64_1_v2[@]}" )
charlist64_2=( "${charlist64_2_v2[@]}" )
# legacy lists, see below

# conventional base64 charset
base64_charset=(
	                                  # dez     hex     oct
	"A" "B" "C" "D" "E" "F" "G" "H"   # 00..07  00..07  00..07
	"I" "J" "K" "L" "M" "N" "O" "P"   # 08..15  08..0f  10..17
	"Q" "R" "S" "T" "U" "V" "W" "X"   # 16..23  10..17  20..27
	"Y" "Z" "a" "b" "c" "d" "e" "f"   # 24..31  18..1f  30..37
	"g" "h" "i" "j" "k" "l" "m" "n"   # 32..39  20..27  40..47
	"o" "p" "q" "r" "s" "t" "u" "v"   # 40..47  28..2f  50..57
	"w" "x" "y" "z" "0" "1" "2" "3"   # 48..55  30..37  60..67
	"4" "5" "6" "7" "8" "9" "+" "/"   # 56..63  38..3f  70..77
)

# only for building new charlist to compare with old ones
compare_charlist_versions(){
	[[ -z $3 ]] \
		&& echo "usage:  $FUNCNAME <list_array_1> <list_array_2_1> <list_array_2_2> [-v]" \
		&& echo "        give three non empty list-array names as arguments. Example:" \
		&& echo "        $FUNCNAME old_arr new_1 new_2" \
		&& return 1
	[[ "$4" == "-v" ]] && verbose=1 || verbose=0
	# pass arrays by name reference
	local -n list_arr_old=$1   2>/dev/null
	local -n list_arr_new_1=$2 2>/dev/null
	local -n list_arr_new_2=$3 2>/dev/null
	# check if arrays are set
	# if ! declare -F "$1" 1>/dev/null \
	# || ! declare -F "$2" 1>/dev/null \
	# || ! declare -F "$3" 1>/dev/null \
	# ; then
	# 	return 1
	# fi
	for list in $1 $2 $3 ; do
		if ! declare -p "$list" > /dev/null 2>&1 ; then
			p_err "given list not declared: $list"
			return 1
		fi
	done
	local i j list new_list_char
	found_difference=0
	for i in {0..63}; do
		j=0 ; list=1
		while [[ "${list_arr_old[i]}" != "${list_arr_new_1[j]}" ]] && (( j<64 )); do
			((j++))
		done
		new_list_char=${list_arr_new_1[j]}
		if (( j==64 )); then
			j=0
			while [[ "${list_arr_old[i]}" != "${list_arr_new_2[j]}" ]] && (( j<64 )); do
				((j++))
			done
			new_list_char=${list_arr_new_2[j]}
			list=2
		fi
		if (( j < 64 )); then
			if (( verbose )); then
				printf "%02d: '%s' %s %02d: '%s'\n" "$i" "${list_arr_old[i]}" "$list" "$j" "$new_list_char"
			fi
		else
			printf "[NOT FOUND IN LIST %s %s] %02d: '%s'\n" "$2" "$3" "$i" "${list_arr_old[i]}"
			found_difference=1
		fi
	done
	if (( ! found_difference )); then
		(( verbose )) && echo
		printf "List [0] is fully represented in lists [1] and [2]\n"
		for i in {1..3} ; do printf "\t[%d] '%s'\n" $((i-1)) "${!i}" ; done
	#else echo "List $1 IS NOT fully represented." ;
	fi
	(( ! verbose )) && echo "(add '-v' to arguments, to see full list.)"
	return 0
}

charlist64_1_legacy=(
	                                  # dez     hex
	"__NULL__"                        # 0       00
	    " " "&" "(" ")" "*" "+" ","   # 01..07  01..07
	"." "/" "0" "1" "2" "3" "4" "5"   # 08..15  08..0f
	"6" "7" "8" "9" ":" "A" "D" "E"   # 16..23  10..17
	"G" "H" "I" "K" "L" "N" "P" "R"   # 24..31  18..1f
	"S" "W" "X" "Z" "a" "b" "c" "d"   # 32..39  20..27
	"e" "f" "g" "h" "i" "j" "k" "l"   # 40..47  28..2f
	"m" "n" "o" "p" "q" "r" "s" "t"   # 48..55  30..37
	"u" "v" "w" "x" "y" "z"           # 56..61  38..3d
	"__use_charlist64_2__"            # 62      3e
	"__use_UTF8__"                    # 63      3f
)
# charlist extension to get all printable ASCII chars and some often used UTF8
charlist64_2_legacy=(
	                                  # dez     hex
	"__NULL_cARR2_unused__"           # 0       00
	# 0x00 unused to reduce confusion with charlist64_1 NULL when decoding manually
	    "ä" "ö" "ü" "Ä" "Ö" "Ü" "ß"   # 01..07  01..07
	"B" "C" "F" "J" "M" "O" "Q" "T"   # 08..15  08..0f
	"U" "V" "Y" "!" "\"" "#" "$" "%"  # 16..23  10..17
	"'" "-" ";" "<" "=" ">" "?" "@"   # 24..31  18..1f
	"[" "\\" "]" "^" "_" "\`" "{" "|" # 32..39  20..27
	"}" "~" "€" "á" "à" "é" "è" "î"   # 40..47  28..2f
	"ø" "Ø" "ú" "ù" "ç" "Ç" "Á" "À"   # 48..55  30..37
	"É" "È" "Î" "Ú" "Ù" "°" "∑" "©"   # 56..63  38..3f
)
