# encode given Indirect Passphrase to a multiple of a 18 Byte string
# using the Date Method.
#
# arguments:
#    '$1'  isSubsequentIPHR  set to '1' if it is a subsequent KeyID
# needed predefined vars are
#    'unencoded_IPHR_input'    as the wellformed data to encode with this method
indirectVar=""
hexadesIPHR=()
tmp_iphrArray=()

# calling first encode_dateMethod_getIPHRs then append tmp_iphrArray to iphrArray
encode_dateMethod(){
	! encode_dateMethod_getIPHRs 0   \
		&& debugMsg "error returned from 'encode_dateMethod_getIPHRs()'"   \
		&& return 1
	! addIPHRto_iphrArray   \
		&& debugMsg "error returned from 'addIPHRto_iphrArray()'"   \
		&& return 1
	return 0
}

# actual encoding of the input. Fills tmp_iphrArray
encode_dateMethod_getIPHRs(){
	# check valid argument
	! [[ $1 == [0-1] ]] && p_err "isSubsequentIPHR wrong format '$1'" && return 2
	isSubsequentIPHR=$1  # Bit 2 of Settings Word
	methodBits=0 # Date Method has Method ID '0x0' ; Bits 3-6 of Settings Word
	
	hexadesIPHR=() # empty array from previous calls or
	               # decoding calls of 'fillArr_hexadesIPHR()'
	               # (for interactive mode)
	# init elememts 1, 2 of arr with placeholders that 3rd elements index is corret
	hexadesIPHR[0]=0 # placeholder for Settings Word
	hexadesIPHR[1]=63 # placeholder for length

	# fill arr with given arguments
	arr_unencoded_IPHR_input=( $(echo "$unencoded_IPHR_input" | awk -F ',' '{for(i=1; i<=NF; i++){print $i}}') )
	if (( debug )); then
		echo "[DEBUG]"
		for((i=0;i<${#arr_unencoded_IPHR_input[@]}; i++));do
			printf "\t[%d] %s\n" $i ${arr_unencoded_IPHR_input[i]}
		done
	fi

	dateMthdLen=0 # Counting segments of Indirect Keydata. 
	              # Equivalent to 'len' encoded in hexadesIPHR[1] 
				  #   (see function 'decIPHRdateMethod()')
	              # One date is one segment '1', 
				  #   one or multiple chars terminated by NULL hexade is one segment '0'.
				  # That is multiple successive '1's can occure but '0's are all solitary
	sbsqtKID=0
	maxDtMthdLen=6
	# iterate over arguments array and encode it
	for((i=0; i<${#arr_unencoded_IPHR_input[@]} & ${#hexadesIPHR[@]}<25 & dateMthdLen<maxDtMthdLen ; i++)); do

		(( ${#hexadesIPHR[@]}+1>24)) && sbsqtKID=1 && debugMsg "all: exceeded blocklen: break" && break

		if [[ "${arr_unencoded_IPHR_input[i]}" == +([0-9]).+([0-9]).+([0-9]) ]]; then
		# if arg is formatted as a date
			(( ${#hexadesIPHR[@]}+5>24)) && sbsqtKID=1 && debugMsg "date: exceeded blocklen: break" && break

			# get encoded Date
			! encodeHexDate ${arr_unencoded_IPHR_input[i]} && return 1
			debugMsg "dateHex is: $(printf "%04x" $(( 0x$dateHex)) )"
			
			# inc i to get next element: word count, check if in range
			(( i++ ))
			! checkValidWordCount "${arr_unencoded_IPHR_input[i]}" && return 1

			# 3 Bytes, 4 hexade words build date and word count
			quadword=$(( 0x$dateHex << 8 | ( ${arr_unencoded_IPHR_input[i]} & 0xff ) ))
			# printf "%06x" $(( $quadword )) | xxd -p -r | xxd -b

			# fill hexadesIPHR
			for (( j=0; j<4; j++ )); do
				hexadesIPHR[${#hexadesIPHR[@]}]=$(( ( ( $quadword >> (6*(3-j) ) ) & 0x00003f) ))
			done

			debugMsg "quadword:" "$(printf "0x%06x\n" $(( $quadword )) )" \
				"$(printf "0o%02o%02o%02o%02o" ${hexadesIPHR[@]:${#hexadesIPHR[@]}-4:4})"

			# shift left the length word and add a '1' for 'date'
			shiftLeft hexadesIPHR[1] 1 63
		elif [[ "${arr_unencoded_IPHR_input[i]}" == +([0-9]) ]]; then
		# else if arg is formatted as a char
			#(( terminateStr )) && (( charCount++ )) # ugly. to keep loop running
			! checkCharInRange "${arr_unencoded_IPHR_input[i]}" && return 2
			
			hexadesIPHR[${#hexadesIPHR[@]}]=${arr_unencoded_IPHR_input[i]}
			debugMsg "$(printf "char: %02d" ${hexadesIPHR[${#hexadesIPHR[@]}-1]})"
			while [[ ${arr_unencoded_IPHR_input[i+1]} == +([0-9]) ]]   \
			&& (( i<${#arr_unencoded_IPHR_input[@]} ))                \
			&& (( ${#hexadesIPHR[@]}<24 )); do
			#already in loop condition: && (( dateMthdLen<maxDtMthdLen )); do
				((i++))
				! checkCharInRange "${arr_unencoded_IPHR_input[i]}" && return 2
				hexadesIPHR[${#hexadesIPHR[@]}]=${arr_unencoded_IPHR_input[i]}
				debugMsg "$(printf "char: %02d" ${hexadesIPHR[${#hexadesIPHR[@]}-1]})"
			done
			
			# terminate char string with NULL word
			(( ${#hexadesIPHR[@]}<24 )) && hexadesIPHR[${#hexadesIPHR[@]}]=0
			
			# shift left the length word and add a '0' for 'char'
			shiftLeft hexadesIPHR[1] 0 63

			# if method length is 4 or 5, length word is after this char is at least 'xxxx0'
			# so no date can get appended to method length word, since it must be terminated by '0[1*]'
			if (( dateMthdLen > 3 )); then
				debugMsg "break for method len reason."
				(( dateMthdLen++, i++ )) # to count arr_unencoded_IPHR_input element, need i++ when break
				break
			fi
		else
			p_message "Encoding method '${encMethod_names_Arr[encMethod]}'."  \
			          "Wrong format: '${arr_unencoded_IPHR_input[i]}'"  \
			          "Valid elements are '<num>' with numbers (1..63) or '$encoding_format_date'"  \
			          "Format has to be comma separated elements '<element>, ...,<element>'"
			return 1
		fi
		
		(( dateMthdLen++ ))
		debugMsg "dateMthdLen: $dateMthdLen - i: '$i'"
	
	done

	if (( i < ${#arr_unencoded_IPHR_input[@]} )); then
		sbsqtKID=1
		left_unenc_IPHR_arr_idx=$i
	else
		left_unenc_IPHR_arr_idx=${#arr_unencoded_IPHR_input[@]}
	fi
	#(( i < ${#arr_unencoded_IPHR_input[@]} )) && debugMsg "+++++subsequent keyID set+++++" || debugMsg "sbsqtKID: '$sbsqtKID'"

	# if last element was not a char / string, shift a '0' in to terminate len Setting Word
	if (( ( ${hexadesIPHR[1]} & 0x1 == 1 ) & dateMthdLen<maxDtMthdLen )) ;then
		shiftLeft hexadesIPHR[1] 0 63
		(( maxDtMthdLen-- )) # shift 1 less if last encoded element is a date,
		                      # because terminating NULL char
		if (( ${#hexadesIPHR[@]} < 24 )); then # terminate with NULL word if block is not full
			hexadesIPHR[${#hexadesIPHR[@]}]=0
		fi
	fi
	for((i=dateMthdLen; i<maxDtMthdLen; i++)); do
		shiftLeft hexadesIPHR[1] 1 63
	done
	
	# write settings to hexadesIPHR, converts to hex iphr; adds to tmp_iphrArray
	transform_hexadesIPHR_to_hex_iphr

	# if subsequent key needed, recurse:
	if (( sbsqtKID )); then
		# set new 'unencoded_IPHR_input' to the leftover data to encode in 'arr_unencoded_IPHR_input'
		debugMsg "sbsqtKID set. recurse into 'encode_dateMethod_getIPHRs()'" \
		         "first of left elements: ${arr_unencoded_IPHR_input[left_unenc_IPHR_arr_idx]}"
		unencoded_IPHR_input=$(
			for(( i=$left_unenc_IPHR_arr_idx; i<${#arr_unencoded_IPHR_input[@]}; i++ ));do
				printf "%s" ${arr_unencoded_IPHR_input[i]}
				if [[ "${arr_unencoded_IPHR_input[i]}" == +([0-9]).+([0-9]).+([0-9]) ]]; then
				# if date, print next element
					((i++))
					printf " %s" ${arr_unencoded_IPHR_input[i]}
				fi
				printf "," # separator
			done
		)
		# recurse to encode the rest
		indirectVar=""
		! encode_dateMethod_getIPHRs 1 && return 1
	fi
	#for((i=0;i<${#arr_unencoded_IPHR_input[@]}; i++));do
#	printf "[%d] %s\t" $i ${arr_unencoded_IPHR_input[i]}
	#	(( (i+1)%4 == 0)) && echo
	#done
	#echo

	return 0
}

# test if char to encode is in range 1..63
checkCharInRange(){
	testChar="$1"
	if ! (( testChar > 0 & testChar < 64 )); then
		p_message "Char number not in valid range (1..63): '$testChar'" && return 1
	fi
}

# Shift a variable (or array element) left one bit, and either ORing a '1' in or 
# leave the '0' as the new least signifant bit in
# Usage preparation before call:
# args: 
#    '$1'   Name of 'variable' or 'arrElement[idx]' that should be shifted
#    '$2'   '1' | '0' new least significant bit
#    '$3'   '[num]' Bit width of new value; ANDed with this value after shifting
shiftLeft(){
	indirectVar="$1"
	insertionBit="$2"
	bitWidth="$3"
	[[ "$indirectVar" == "" ]] \
		&& p_err "indirectVar empty" && return 2
	! [[ $insertionBit == [0-1] ]] \
		&& p_err "invalid insertionBit" && return 2
	! [[ $bitWidth == +([0-9]) ]] \
		&& p_err "invalid bitWidth" && return 2
	if (( insertionBit )); then
		shiftValue=$(( ( ( ${indirectVar} << 1 | 0x01 ) & bitWidth ) )) # shift a '1' in
	else
		shiftValue=$(( ( ( ${indirectVar} << 1 ) & bitWidth ) )) # shift a '0' in
	fi
	declare -g $indirectVar=$shiftValue
	debugMsg "shift length: $(printf "%02x" "${!indirectVar}" | xxd -r -p | xxd -b | cut -d ' ' -f 2 )"
	return 0
}
checkValidWordCount(){
	# from https://stackoverflow.com/questions/806906/how-do-i-test-if-a-variable-is-a-number-in-bash
	! [[ $1 == ?(-)+([0-9]) ]] 	\
		&& p_message "Dates word count has to be a number" && return 1
	(( $1 == 0 )) 	\
		&& p_message "Dates word count can't be 0" && return 1
	! (( $1 < 128 & $1 > -129)) 	\
		&& p_message "Dates 'word count' has to be in range -128 to 127, not 0)"  \
		&& return 1
	return 0
}

