# openssl decrypt database of IPHRs,
# fills array 'iphrArray' with the IPHRs,
# checks consistency / correct DB passphrase by md5
# of all IPHRs and comparison with last array
# element (which is this md5)
iphrArray=()
iphrArray_isDecrypted=0
keyDatabaseSalt=""
keysAssignmentList=""
decrypt_iphrArrayDB(){
	if test -f "$keyDatabase";then
		if (( ! iphrArray_isDecrypted )); then
			read_passphrase
		else
			return 0
		fi
		iphrArray=(
			 $(openssl enc -d -aes-128-ctr -pbkdf2 -pass stdin -in "$keyDatabase" <<< "$db_passphrase" | xxd -p -c 18)
		)
		keyDatabaseSalt=$(xxd -p -c 8 -l 16 "$keyDatabase" | tail -n 1)
	else
		(( opt_decode | opt_list )) && p_message "No key database found. (path is: '$keyDatabase'" && return 1
		create_database && return 0 || return 1
	fi
	debugMsg "in $FUNCNAME" "keyDatabaseSalt: '$keyDatabaseSalt'"
	# remove md5 from array
	if (( ${#iphrArray[@]} > 0 )); then
		iphrArrMD5_lastArrElement=${iphrArray[${#iphrArray[@]}-1]}
		unset iphrArray[${#iphrArray[@]}-1]
	fi

	get_iphrArrMD5
	if (( ${#iphrArray[@]} == 0 )) \
	|| [[ "$iphrArrMD5_lastArrElement" != "$iphrArrMD5" ]]; then
		if (( debug )); then
			if (( ${#iphrArray[@]} == 0 )); then
				debugMsg "no decryption. array empty"
			else
			 	debugMsg "iphrArrMD5_lastArrElement:" \
				         "$iphrArrMD5_lastArrElement" \
				         "$(
				         	for (( i=0; i<${#iphrArray[@]}; i++ )); do
				         		(( i>0 )) && printf "\t"
				         		printf "%s\n" ${iphrArray[i]}
				         done)"
			fi
		fi
		p_message "Wrong passphrase" && return 1
	fi
	keysAssignmentList=$(openssl enc -d -aes-128-ctr -pbkdf2 -pass stdin -in "$keysAssignmentFile" <<< "$db_passphrase")

	debugMsg "END $FUNCNAME" "iphrArray (${#iphrArray[@]} elements):" "$(printf "%s\n" ${iphrArray[0]}; printf "\t%s\n" ${iphrArray[@]:1})"
	iphrArray_isDecrypted=1
	return 0
}
create_database(){
	printf "key database not found. Create? (y/n) "
	read -n 1 answer
	echo
	if [[ "$answer" != "y" ]]; then
		return 1
	fi
	touch "$keyDatabase"
	touch "$keysAssignmentFile"
	iphrArray=()
	iphrArray_isDecrypted=1
	db_passphrase_check=""
	read_passphrase
	printf "Enter passphrase again: "
	IFS= read -rs db_passphrase_check < /dev/tty
	while [[ "$db_passphrase" != "$db_passphrase_check" ]]; do
		echo
		echo "Your two inputs are not matching. Please try again:"
		read_passphrase
		printf "Enter passphrase again: "
		IFS= read -rs db_passphrase_check < /dev/tty
	done
	echo
	# create empty keyDatabase
	! ( openssl enc -e -aes-128-ctr -pbkdf2 -pass stdin -out "$keyDatabase" -in <(printf "" | xxd -p -r) ) <<< "$db_passphrase"  \
		&& p_err "openssl error. Key database not created."   \
		&& rm "$keyDatabase"                                  \
		&& rm "$keysAssignmentFile"                           \
		&& return 1
	keyDatabaseSalt=$(xxd -p -c 8 -l 16 "$keyDatabase" | tail -n 1)
	keysAssignmentList="$(printf "Keys Assignment list\n")"
	return 0
}
