# Parse options.
# Return values are:
#    0  if execution should continue
#    1  if execution should stop
opt_encode=0
opt_decode=0
opt_list=0
opt_interactive=0
keyID=-1
debug=0
encMethodArr=( 0 1 2 ) # valid methods
encMethod_names_Arr=( "date/charnum" "XOR_charlist64" "charlist64" ) # valid methods
encMethod=${encMethodArr[0]} # default is Date Method (also usable as charlist64)
unencoded_IPHR_input=""
fail=0
directDecode=0 # if IPHR is given as an argument
parseOptions(){
	if [ $# -lt 1 ]; then
		echo "Decode or encode Indirect Passphrases (IPHR)."
		usage
	fi
	while [ $# -gt 0 ]; do
		debugMsg "parse arg: '$1'"
		case $1 in
			--debug|-dbg)
				(( debug )) && debug=0 || debug=1
				debugMsg "debugging on"
			;;
			-d|--direct)
				[[ "$2" == "" ]] && (( !opt_interactive ))  \
					&& p_message "Option '$1' needs an argument"  \
					&& usage && return 1
				IPHR_input=${2}
				(( opt_interactive )) && printf "Enter IPHR: " && IFS=$'\n' read IPHR_input
				IPHR_input=$(echo "${IPHR_input}" | tr -d ' ') # remove spaces if grouped
				iphrArray=( "$IPHR_input" )
				directDecode=1
				opt_decode=1
				shift
				# support for octal argument
				if [[ "${IPHR_input:0:2}" == "0o" ]]; then
					getOctalIPHR_input || return 1
				else
					warn_if_octalIPHR
					! checkHex ${iphrArray[0]} && p_message "Input has wrong format: '$iphrArray'" && return 1
					! checkArgLen ${iphrArray[0]} 36 && p_message "Input was not 18 byte: '$iphrArray'" && return 1
				fi
			;;
			-cdb|--choose-database)
				choose_database || return 1
			;;
			-[0-9]*)
				p_message "keyID has to be a positive integer number."
				(( !opt_interactive )) && usage
				return 1
			;;
			[0-9]*)
				if ! checkValid_keyID $1; then
					(( !opt_interactive )) && usage
					return 1
				fi
				opt_decode=1
			;;
			-l|--list)
				# list assignments
				opt_list=1
			;;
			-h|--help)
				usage
				optlist
				return 0
			;;
			
			-e|--encode)
				# -e, --encode <indirect passphrase>
				#    Encode your indirect passphrase.
				#    Method IDs are (default: 0)
				#       0  Date Method (or charlist64)
				#       1  wordlist

				debugMsg "encode mode"
				opt_encode=1
				#[[ "$2" == "" ]] && p_message "Option '$1' needs two arguments" && usage && return 1
				#unencoded_IPHR_input="$2"
				# shift
			;;

			-i|--interactive)
				(( opt_interactive )) && p_message "opt_interactive already set. use 'q' to quit."
				opt_interactive=1
			;;
			
			-se|--switch-screenerase)
				(( !opt_interactive )) && echo "option '$1' only useful in interactive mode." \
				                       && shift && continue
				(( screenErase )) && switch_screenErase 0 || switch_screenErase 1
				(( !opt_switch_screenerase & !screenErase )) && opt_switch_screenerase=1 || opt_switch_screenerase=0
			;;

			-m|--method)
				encMethod=$2 ; (( encMethod-- )) # decrement, starting at ID '0'
				! [[ " ${encMethodArr[@]} " =~ " $encMethod " ]] && p_message "Invalid Method ID for option '$1'." "see: --help" && return 1
				echo "Encoding method set to '${encMethod_names_Arr[$encMethod]}'"
				(( opt_interactive )) && sleep 3
				shift
			;;

			-v|--version)
				echo "Version $version"
				return 0 # output version and end options parsing
			;;
			
			*)
				(( opt_interactive ))   \
					&& p_message "Not an option. '${1:1}'"   \
					&& return 1
				p_message "Not an option: '$1'"
				usage
				return 1
			;;
		esac
		shift
	done
	(( opt_decode & opt_encode )) && p_message "Giving a <keyID> or set option '-d, --direct' is not usable with '-e, --encode'." && usage && return 1
	return 0
}

warn_if_octalIPHR(){
	# give warning if octal is detected, to use prefix for octal input
	if checkOct $IPHR_input; then
		p_message "[INFO]  Octal number detected."      \
	         "In case you wanted octal input, use '0o' prefix."
		(( opt_interactive )) && sleep 3
	fi
	return 0
}
getOctalIPHR_input(){
	oct_IPHRinput=${IPHR_input:2}
	! checkOct $oct_IPHRinput && p_message "input was not octal: '$oct_IPHRinput'" && return 1
	(( ${#oct_IPHRinput} % 2 != 0 ))                \
	   && p_message "Octal number doesn't have even digits: '$oct_IPHRinput'"  \
	   && return 1
	# fill input into hexadesIPHR array
	tmp_iphrArray=() # empty for interactive mode
	for (( i=0; i < ${#oct_IPHRinput}; i+=2 )); do
		hexadesIPHR[i/2]=$(( ${oct_IPHRinput:i:1}*8 + ${oct_IPHRinput:i+1:1} ))
	done
	sbsqtKID=$((         ( ${hexadesIPHR[0]} & 0x20 ) >> 5 ))
	isSubsequentIPHR=$(( ( ${hexadesIPHR[0]} & 0x10 ) >> 4 ))
	methodBits=$((       ( ${hexadesIPHR[0]} & 0x0f ) >> 0 ))
	! transform_hexadesIPHR_to_hex_iphr && p_err "returning from transform_hexadesIPHR_to_hex_iphr" && return 1
	iphrArray=( "${tmp_iphrArray}" )
	return 0
}
